﻿Option Explicit On

Module MdlMikapen

  '定数
  Public Const WEB_URL = "http://milltype.com/mikapen/" 'ホームページ
  Public Const EX_NAME = ".mpj" 'ミカペンプロジェクトファイル用拡張子の名前
  Public Const LAYER_NAME = "_Layer" 'レイヤーファイル保存用のフォルダ名に付加する名前
  Public Const NON_TITLE = "無題" 'プロジェクト名未入力時の名前

  'ペンとブラシ
  Public gPenLeft As Pen    '左ドラッグ時に描画用のペン
  Public gPenRight As Pen   '右ドラッグ時に描画用のペン
  Public gPenErase As Pen   '消去用のペン
  Public gPenBlackW1 As Pen '黒の太さ1のペン
  Public gPenEraseW1 As Pen '消去用の太さ1のペン
  Public gCurrentPen As Pen 'カレントのペン

  Public gSolidBrushLeft As SolidBrush    '左ドラッグ時に描画用のブラシ
  Public gSolidBrushRight As SolidBrush   '右ドラッグ時に描画用のブラシ
  Public gSolidBrushErase As SolidBrush   '消去用のブラシ
  Public gCurrentSolidBrush As SolidBrush 'カレントのブラシ

  Public gColorLeft As Color           '左ドラッグ時の色
  Public gColorRight As Color          '右ドラッグ時の色
  Public gColorErase As Color          '消去用の色
  Public gBmpTransparentColor As Color 'ビットマップの透過色

  'フラグや、一時的な変数
  Public gWritingFlg As Boolean '描画中(マウスのドラッグ中)ならTrue
  Public gUpdFlg As Boolean     '何か更新して、保存前ならTrue
  Public gDrawMode As Integer   'エンピツ:1、消しゴム:2、やり直し:3、直線:4、四角:5、円:6
  Public gPenWidth As Integer   'ペンの幅
  Public gPenTrans As Integer   'ペンの透明度
  Public gMouseBaseX As Integer 'マウスの基準点x位置
  Public gMouseBaseY As Integer 'マウスの基準点y位置
  Public gMouseFromX As Integer 'マウス移動先のx位置
  Public gMouseFromY As Integer 'マウス移動先のy位置

  '列挙型
  Public Enum DrawMode As Integer
    modeNothing = 0
    modePencil = 1  'エンピツ
    modeEraser = 2  '消しゴム
    modeUnDo = 3    'やり直し
    modeLine = 4    '直線
    modeSquare = 5  '四角
    modeCircle = 6  '円
  End Enum
  '**************************************************
  ' 設定された色から、ペンやブラシを作成
  '**************************************************
  Public Sub ColorSet()

    If Not IsNothing(gPenLeft) Then gPenLeft.Dispose()
    If Not IsNothing(gPenRight) Then gPenRight.Dispose()
    If Not IsNothing(gPenErase) Then gPenErase.Dispose()
    If Not IsNothing(gPenBlackW1) Then gPenBlackW1.Dispose()
    If Not IsNothing(gPenEraseW1) Then gPenEraseW1.Dispose()
    If Not IsNothing(gSolidBrushLeft) Then gSolidBrushLeft.Dispose()
    If Not IsNothing(gSolidBrushRight) Then gSolidBrushRight.Dispose()
    If Not IsNothing(gSolidBrushErase) Then gSolidBrushErase.Dispose()

    gPenLeft = New Pen(Color.FromArgb(gPenTrans, gColorLeft), gPenWidth)
    gSolidBrushLeft = New SolidBrush(Color.FromArgb(gPenTrans, gColorLeft))

    gPenRight = New Pen(Color.FromArgb(gPenTrans, gColorRight), gPenWidth)
    gSolidBrushRight = New SolidBrush(Color.FromArgb(gPenTrans, gColorRight))

    gPenErase = New Pen(gColorErase, gPenWidth)
    gSolidBrushErase = New SolidBrush(gColorErase)

    gPenBlackW1 = New Pen(Color.Black, 1)
    gPenEraseW1 = New Pen(Color.FromArgb(0, Color.Transparent), 1)

    gBmpTransparentColor = System.Drawing.ColorTranslator.FromOle(RGB(255, 255, 255))

  End Sub
  '**************************************************
  ' どの方法で保存するかを調べる
  ' 戻り値 True :保存を実行または、保存しないを選んだとき
  '        False:キャンセルしたとき -> プログラムは終了しないように
  '**************************************************
  Public Function SaveCheck() As Boolean
    Dim result As DialogResult

    If gUpdFlg = True Then
      '保存前の修正があるとき
      result = MessageBox.Show(gProjectName & " への変更を保存しますか？", Application.ProductName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1)
      If result = DialogResult.Yes Then
        If gProjectName = NON_TITLE Then
          '名前を付けて保存
          If ProjectSaveNameRtn() = True Then
            Return True
          Else
            Return False
          End If
        Else
          '上書き保存
          ProjectSaveRtn()
          Return True
        End If
      End If
      If result = DialogResult.No Then
        Return True
      End If
      If result = DialogResult.Cancel Then
        Return False
      End If
      Return False
    Else
      Return True
    End If

  End Function
  '**************************************************
  ' クローズ時の後始末
  '**************************************************
  Public Sub CloseRtn()
    Dim i As Integer

    FrmMikapen.WindowState = FormWindowState.Normal
    My.Settings.Size = FrmMikapen.Size

    For i = 1 To gLayerNum
      RemoveHandler gTextBoxLayerName(i).Click, AddressOf FrmMikapen.mTextBoxLayerName_Click
      RemoveHandler gTextBoxLayerName(i).DoubleClick, AddressOf FrmMikapen.mTextBoxLayerName_DoubleClick
      RemoveHandler gTextBoxLayerName(i).KeyDown, AddressOf FrmMikapen.mTextBoxLayerName_KeyDown
      RemoveHandler gTextBoxLayerName(i).Leave, AddressOf FrmMikapen.mTextBoxLayerName_Leave
      RemoveHandler gPictureBoxLayer(i).Click, AddressOf FrmMikapen.mPictureboxLayer_Click
      RemoveHandler gPanelLayer(i).Click, AddressOf FrmMikapen.mPanelLayer_Click

      If Not IsNothing(gPanelLayer(gLayerNum)) Then gPanelLayer(gLayerNum).Dispose()
      If Not IsNothing(gPictureBoxLayer(gLayerNum)) Then gPictureBoxLayer(gLayerNum).Dispose()
      If Not IsNothing(gTextBoxLayerName(gLayerNum)) Then gTextBoxLayerName(gLayerNum).Dispose()
      If Not IsNothing(gBitmap(gLayerNum)) Then gBitmap(gLayerNum).Dispose()

      FrmMikapen.PanelFront.Controls.Remove(gPanelLayer(gLayerNum))
      gPanelLayer(gLayerNum).Controls.Remove(gPictureBoxLayer(gLayerNum))
      gPanelLayer(gLayerNum).Controls.Remove(gTextBoxLayerName(gLayerNum))
    Next i

    'リソースを開放する
    If Not IsNothing(gPenLeft) Then gPenLeft.Dispose()
    If Not IsNothing(gPenRight) Then gPenRight.Dispose()
    If Not IsNothing(gPenErase) Then gPenErase.Dispose()
    If Not IsNothing(gPenBlackW1) Then gPenBlackW1.Dispose()
    If Not IsNothing(gPenEraseW1) Then gPenEraseW1.Dispose()
    If Not IsNothing(gCurrentPen) Then gCurrentPen.Dispose()
    If Not IsNothing(gSolidBrushLeft) Then gSolidBrushLeft.Dispose()
    If Not IsNothing(gSolidBrushRight) Then gSolidBrushRight.Dispose()
    If Not IsNothing(gSolidBrushErase) Then gSolidBrushErase.Dispose()
    If Not IsNothing(gCurrentSolidBrush) Then gCurrentSolidBrush.Dispose()

    If Not IsNothing(gBitmapDraw) Then gBitmapDraw.Dispose()

    If Not IsNothing(gGraphicsTop) Then gGraphicsTop.Dispose()
    If Not IsNothing(gGraphicsDraw) Then gGraphicsDraw.Dispose()
    If Not IsNothing(gGraphicsWork) Then gGraphicsWork.Dispose()

  End Sub
  '**************************************************
  ' ピクチャーボックスの大きさを設定
  '**************************************************
  Public Sub PictureBoxSizeSet()

    'ピクチャーパネルの位置変更
    FrmMikapen.PanelPictureBack.AutoScrollPosition = New Point(0, 0)

    '各ピクチャーボックスの位置と大きさを設定
    FrmMikapen.PictureBoxTop.Width = My.Settings.CanvasWidth
    FrmMikapen.PictureBoxTop.Height = My.Settings.CanvasHeight
    FrmMikapen.PictureBoxTop.Location = New Point(0, 0)

    FrmMikapen.PictureBoxLayerFront.Width = My.Settings.CanvasWidth
    FrmMikapen.PictureBoxLayerFront.Height = My.Settings.CanvasHeight
    FrmMikapen.PictureBoxLayerFront.Location = New Point(0, 0)

    FrmMikapen.PictureBoxDraw.Width = My.Settings.CanvasWidth
    FrmMikapen.PictureBoxDraw.Height = My.Settings.CanvasHeight
    FrmMikapen.PictureBoxDraw.Location = New Point(0, 0)

    FrmMikapen.PictureBoxWork.Width = My.Settings.CanvasWidth
    FrmMikapen.PictureBoxWork.Height = My.Settings.CanvasHeight
    FrmMikapen.PictureBoxWork.Location = New Point(0, 0)

    FrmMikapen.PictureBoxLayerBack.Width = My.Settings.CanvasWidth
    FrmMikapen.PictureBoxLayerBack.Height = My.Settings.CanvasHeight
    FrmMikapen.PictureBoxLayerBack.Location = New Point(0, 0)

    If Not IsNothing(gBitmapDraw) Then gBitmapDraw.Dispose()
    'ピクチャーボックスとビットマップの関連付け
    '描画用のPictureBoxDrawのみ独立したビットマップを持つ
    gBitmapDraw = New Bitmap(My.Settings.CanvasWidth, My.Settings.CanvasHeight)
    FrmMikapen.PictureBoxDraw.Image = Nothing
    FrmMikapen.PictureBoxDraw.Image = gBitmapDraw

    FrmMikapen.PictureBoxTop.Image = Nothing
    FrmMikapen.PictureBoxTop.Image = New Bitmap(My.Settings.CanvasWidth, My.Settings.CanvasHeight)

    FrmMikapen.PictureBoxLayerFront.Image = Nothing
    FrmMikapen.PictureBoxLayerFront.Image = New Bitmap(My.Settings.CanvasWidth, My.Settings.CanvasHeight)

    FrmMikapen.PictureBoxWork.Image = Nothing
    FrmMikapen.PictureBoxWork.Image = New Bitmap(My.Settings.CanvasWidth, My.Settings.CanvasHeight)

    FrmMikapen.PictureBoxLayerBack.Image = Nothing
    FrmMikapen.PictureBoxLayerBack.Image = New Bitmap(My.Settings.CanvasWidth, My.Settings.CanvasHeight)

    'pictureBoxの背景を設定
    gGraphicsTop = Graphics.FromImage(FrmMikapen.PictureBoxTop.Image)
    gGraphicsDraw = Graphics.FromImage(FrmMikapen.PictureBoxDraw.Image)
    gGraphicsWork = Graphics.FromImage(FrmMikapen.PictureBoxWork.Image)

  End Sub

End Module
